#include <stdlib.h>
#include <stdio.h>
#include "patoh.h"


#define NCONST        4
#define WDIFF         10

/*
  #define _DEBUG1
*/

#define umax(a, b)	(((a) >= (b)) ? (a) : (b))

#define usRandom(s)     srand(s)
#define uRandom(l)      (rand() % l)



void usage(char *name)
{
 printf("usage: %s <partitioner-char> <hypergraph-filename> <#parts>\n", name);
 printf("\t<partitioner-char> can be:\n");
 printf("\t\tP: Single constrain partition\n");
 printf("\t\tF: with first #parts cells are assigned to each part\n");
 printf("\t\tM: %d-constrain partition\n", NCONST);
 printf("\t\tA: run all of them\n");
 exit(1);
}


void PrintInfo(char *pname, int _k, int *partweights, int cut, int _nconst)
{
 double             *avg, *maxi;
 int                i, j;

 printf("\n-------------------------------------------------------------------");
 printf("\n Partitioner: %s", pname);
 
 printf("\n %d-way cutsize = %d \n", _k, cut);
 
 printf("\nPartWeights are:\n");
 avg = (double *) malloc(sizeof(double)*_nconst);
 maxi = (double *) malloc(sizeof(double)*_nconst);
 for (i=0; i<_nconst; ++i)
     maxi[i] = avg[i] = 0.0;     
 for (i=0; i<_k; ++i)
       for (j=0; j<_nconst; ++j)
         avg[j] += partweights[i*_nconst+j];
  for (i=0; i<_nconst; ++i)
     {
     maxi[i] = 0.0;     
     avg[i] /= (double) _k;
     }
 
 for (i=0; i<_k; ++i)
     {
     printf("\n %3d :", i);
     for (j=0; j<_nconst; ++j)
         {
         double im= (double)((double)partweights[i*_nconst+j] - avg[j]) / avg[j];
         
         maxi[j] = umax(maxi[j], im);
         printf("%10d ", partweights[i*_nconst+j]);
         }     
     }
#ifdef _DEBUG1
 printf("\n Average Weights are:");
 printf("\n      ");
 for (i=0; i<_nconst; ++i)
     printf("%10.1lf ", avg[i]);
#endif
 printf("\n MaxImbals are (as %%):");
 printf("\n      ");
 for (i=0; i<_nconst; ++i)
     printf("%10.1lf ", 100.0*maxi[i]);
 
 usfree(maxi);
 usfree(avg);
}


int main(int argc, char *argv[])
{
 PaToH_Parameters args;
 int             _c, _n, _nconst, *cwghts, *nwghts, 
     *xpins, *pins, *partvec, cut, *partweights,
     i, *cw;
 char           PT;


 if (argc!=4)
     usage(argv[0]);     

 PT = toupper(*argv[1]);
 if (PT!='P' && PT!='F' && PT!='M' && PT!='A')
     usage(argv[0]);
 usRandom(time(NULL));
 
 
 PaToH_Read_Hypergraph(argv[2], &_c, &_n, &_nconst, &cwghts, 
                       &nwghts, &xpins, &pins);

 printf("===============================================================================\n");
 
 printf("Hypergraph %10s -- #Cells=%6d  #Nets=%6d  #Pins=%8d #Const=%2d\n",
        argv[2], _c, _n, xpins[_n], _nconst);
 
 PaToH_Initialize_Parameters(&args, PATOH_CONPART, 
                             PATOH_SUGPARAM_DEFAULT);

 args._k = atoi(argv[3]);

 partvec =  (int *) malloc(_c*sizeof(int));
 partweights  = (int *) malloc(NCONST*args._k*sizeof(int));
 
 PaToH_Alloc(&args, _c, _n, NCONST, cwghts, nwghts, 
             xpins, pins);
 

 if (PT=='P' || PT=='A')
     {     
     PaToH_Partition(&args, _c, _n, cwghts, nwghts, 
                     xpins, pins, partvec, partweights, &cut);
     PrintInfo("Single Constrain", args._k, partweights, cut, 1);
     
     }
 

 if (PT=='F' || PT=='A')
     {
     memset(partvec, 0xff, _c*sizeof(int));
     for (i=0; i<args._k; ++i)
         partvec[i] = i;
     PaToH_Partition_with_FixCells(&args, _c, _n, cwghts, nwghts, 
                                   xpins, pins, partvec, partweights, &cut);
     PrintInfo("With FixCells", args._k, partweights, cut, 1);
     }


 if (PT=='M' || PT=='A')
     {
     _nconst = NCONST;
     
     cw =  (int *) malloc(_c*_nconst*sizeof(int));
     for (i=0; i<_c; ++i)
         {
         int j;

         for (j=0; j<_nconst; ++j)
             {
             int w=cwghts[i]+(uRandom(WDIFF)-WDIFF/2)*j;
             cw[i*_nconst+j] = umax(w, 1);             
             }
         
         }
 
     PaToH_MultiConst_Partition(&args, _c, _n, _nconst, cw, xpins, pins, partvec, partweights, &cut);
     PrintInfo("MultiConstrain", args._k, partweights, cut, _nconst);
     usfree(cw);
     
     }
 

free(partweights);
free(partvec);
  
PaToH_Free();
printf("\n");
 
return 0;
}

